/* GFV
 * Copyright (C) 1999 Sam Lotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "srl_ui_util.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
// #include "dlg_startingDirectory.h"
#include "pref_dlg.h"
#include "find_dlg.h"

#include <sys/stat.h>
#include <sys/types.h>
#include <ctype.h>
#include <errno.h>

GtkWidget * mainApp = NULL;
static char * cDir = NULL;		// Current directory
//static char tbuf[256];

static gchar *viewFile = NULL;
gchar *editName = NULL;


// ------------- GLOBAL VARIABLES FOR ENVIRONMENT -----------------------
gchar *gbl_font_name = NULL;
GdkFont *gbl_font = NULL;



static void killApp(){
  // g_print("in kill app\n");
  if (gbl_font != NULL){
    gdk_font_unref(gbl_font);
  }
  
  gtk_main_quit();                                        
}




/* ---------- STANDARD POPUP MENU 
 *  Respond to a button-press by posting a menu passed in as widget.
 *
 * Note that the "widget" argument is the menu being posted, NOT
 * the button that was pressed.
 */

static gint button_press_popup (GtkWidget *widget, GdkEvent *event)
{

    if (event->type == GDK_BUTTON_RELEASE) {
      GdkEventButton *bevent = (GdkEventButton *) event; 
    
      if (bevent->button == SRL_RIGHT_BUTTON){
        gtk_menu_popup (GTK_MENU(widget), NULL, NULL, NULL, NULL,
                        bevent->button , bevent->time);
        /* Tell calling code that we have handled this event; the buck
         * stops here. */
        return TRUE;
     }
    }

    /* Tell calling code that we have not handled this event; pass it on. */
    return FALSE;
}





/* ----
	Setup the size of the panels ....
****/
static void setupPanelSpacing(){

	GtkAllocation * alloc = NULL;

	GtkHPaned * hp = GTK_HPANED(gtk_object_get_data (GTK_OBJECT (mainApp), "hpaned1"));

	GtkVPaned * vp = GTK_VPANED(gtk_object_get_data (GTK_OBJECT (mainApp), "vpaned1"));

	
	
	// Get the size of the container
	alloc = &(GTK_WIDGET(hp)->allocation);
	
	gtk_paned_set_gutter_size (GTK_PANED(hp), (guint16)10);
	gtk_paned_set_gutter_size (GTK_PANED(vp), (guint16)10);	
	
	gtk_paned_set_position(GTK_PANED(hp),(gint)(alloc->width/4));
	gtk_paned_set_position(GTK_PANED(vp),(gint)(alloc->height/2));
}





// RESIZE OR MOVE event!
gboolean on_GFV_configure_event                 (GtkWidget       *widget,
                                        GdkEventConfigure *event,
                                        gpointer         user_data)
{

  static gint xw=-1, xh=-1;
  //gdk_window_get_size	 (event->window, &xw, &xh);


  if (event->type == GDK_CONFIGURE){
    if ((xw != event->width) ||  (xh != event->height)){
      setupPanelSpacing();
    }  
    xw = event->width;
    xh = event->height;
  } 
  
  return FALSE;
}



static void setupColumns(){

  GtkCList * cl = GTK_CLIST(gtk_object_get_data (GTK_OBJECT (mainApp), "filelist"));
  int x=0;

  
  for (x=0;x<cl->columns;x++){
    // Ok, lets handle the resize of the columns
    // We want to record this info in the config file
    gchar * sKey = NULL;
    gint wdth;
    gboolean bDef = FALSE;
    sKey = g_strdup_printf("settings/FileListCol_%d=-1",x);

    wdth=gnome_config_get_int_with_default(sKey, &bDef);
    if (bDef == FALSE){
      gtk_clist_set_column_width (cl, x, wdth);
    }

    g_free(sKey);
  
  }
  
  

}




void killEntireTree(){
	GtkTree * tree = GTK_TREE(gtk_object_get_data (GTK_OBJECT (mainApp), "tree1"));

	gtk_tree_clear_items(tree,0,-1);
}


// Call each time 
void buildInitialTree(char * base){
	GtkTree * tree = GTK_TREE(gtk_object_get_data (GTK_OBJECT (mainApp), "tree1"));
	GtkTree * st = NULL;
	GtkTreeItem * item = NULL;
	gboolean b;


	
	push_status(_("Building directory tree"));
	setWaitCursor();

	
	item = GTK_TREE_ITEM(gtk_tree_item_new_with_label(base));
	gtk_tree_append( tree, GTK_WIDGET(item));
	gtk_widget_show( GTK_WIDGET(item) );
	
	st = buildTreeFromDir(item,base);
	
	if (gnome_config_get_bool_with_default("settings/expand=false",&b)){
	  gtk_tree_item_expand(item);
	}
	
	gtk_tree_select_item(tree,0);			// Select the first item

	pop_status();
	setNormalCursor();
}



void buildAllTrees(void){
  gchar * initDir = NULL;
  gboolean d;
  gint cnt;
  
  cnt = gnome_config_get_int_with_default("settings/BDCount=0", &d);
  
  killEntireTree();
  if (cnt == 0){
    // old format !
    initDir = gnome_config_get_string_with_default("settings/BASE_DIR=/",&d);
    buildInitialTree(initDir);	
    g_free(initDir);
  } else {
    int x=0;
    char buf[50];
    
    for (x=0;x<cnt;x++){
      sprintf(buf,"settings/BASE_DIR_ENTRY%d",x);
      initDir = gnome_config_get_string(buf);
      buildInitialTree(initDir);	
      g_free(initDir);
    }
  }
}	





void on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   killApp(); // on_gfv_delete();
}


void on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkText * txt = NULL;
	txt = GTK_TEXT(gtk_object_get_data (GTK_OBJECT (mainApp), "txtViewer"));
	
	gtk_editable_copy_clipboard (GTK_EDITABLE(txt));


}



void on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *about2;
  about2 = create_about2 ();
  gnome_dialog_set_parent( GNOME_DIALOG(about2), GTK_WINDOW(mainApp));
  gnome_dialog_run( GNOME_DIALOG(about2) );
  

}



/* ---------------------
   This is where I start my process......
   Gloade opens the window and i take it from the show event 
*/   
void on_gfv_show                            (GtkWidget       *widget,
                                        gpointer         user_data)
{
  gchar * fname = NULL;
  gboolean d;
  
  // gchar * initDir = NULL;
  //gboolean b;
	
  mainApp = widget;
  gnome_config_push_prefix("/gfv/");
	

  editName = gnome_config_get_string_with_default("settings/editName=gedit",&d);

	
  /* Attach popups */
  gtk_signal_connect_object(
    GTK_OBJECT(gtk_object_get_data (GTK_OBJECT (mainApp), "txtViewer")), 
    "button_release_event",
    GTK_SIGNAL_FUNC (button_press_popup), GTK_OBJECT(GTK_MENU(create_textViewMenu())));
  	
  /* Attach popups */
  gtk_signal_connect_object(
    GTK_OBJECT(gtk_object_get_data (GTK_OBJECT (mainApp), "filelist")), 
    "button_release_event",
    GTK_SIGNAL_FUNC (button_press_popup), GTK_OBJECT(GTK_MENU(create_fileListMenu())));
	
	
  buildAllTrees();	
	
  setupPanelSpacing();
  setupColumns();
	

  // Load the font name	
  fname = gnome_config_get_string_with_default("settings/font=NONE",&d);	
  if (!d){
    gbl_font_name = g_strdup(fname);
    gbl_font = gdk_font_load(gbl_font_name);
  }
  g_free(fname);
	
	
}	
	


void on_filelist_select_row                 (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gchar *fname = NULL;
  GtkText * txt = NULL;
  GnomeAppBar * appBar = NULL;

  if (viewFile != NULL) g_free(viewFile);

  gtk_clist_get_text (clist,row,0,&fname);
  viewFile = g_strdup_printf("%s%s", cDir, fname);
	
  txt = GTK_TEXT(gtk_object_get_data (GTK_OBJECT (mainApp), "txtViewer"));

  appBar = GNOME_APPBAR(gtk_object_get_data (GTK_OBJECT (mainApp), "appbar1"));
  gnome_appbar_set_default(appBar, viewFile);

					  
  readFileIntoTextControl(txt , viewFile, gbl_font);

}

// REDO THE VIEW ... For font changes
void refreshFileView(void){
  GtkText * txt = NULL;
  txt = GTK_TEXT(gtk_object_get_data (GTK_OBJECT (mainApp), "txtViewer"));

  if (viewFile != NULL){
    readFileIntoTextControl(txt , viewFile, gbl_font);
  }
}



void
on_testmenu1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}


/* Clear out the clist and add entries for the files */
void refreshFileList(gchar * dirName){
	// Build file list
	
	GtkCList * cl = GTK_CLIST(gtk_object_get_data (GTK_OBJECT (mainApp), "filelist"));

	struct stat * st;
	DIR *dir = opendir(dirName);
	struct dirent *ent = NULL;


	gchar *cols[10];
	int c=0;
	
	push_status(_("Reading directory"));		
	setWaitCursor();
			
	gtk_clist_column_titles_passive (cl);
	gtk_clist_clear (cl);
	gtk_clist_freeze(cl);
	gtk_clist_set_column_justification (cl,2,GTK_JUSTIFY_RIGHT);

	while( (ent = readdir(dir)) != NULL){
	  if (strcmp(ent->d_name,".") == 0) continue;
	  if (strcmp(ent->d_name,"..") == 0) continue;
	  		
	  if (isDirectory2(dirName, ent->d_name)) continue;
	  
	
	  // Lets buiid the columns of information
	  st = getStat2(dirName, ent->d_name);
	  c=0;
	  cols[c++]= g_strdup_printf("%s", ent->d_name);	// Name
	  cols[c++]= g_strdup(ctime(&(st->st_atime))); // Time
	  cols[c++]= g_strdup_printf("%lu", st->st_size); // Size
	  
	  g_free(st);
		
		
	  gtk_clist_append( cl, cols);

  	  for (c=0;c<3;c++){
	    g_free(cols[c]);
	  }
		
	}
	gtk_clist_thaw(cl);

	if (closedir(dir) == -1){
	    fprintf(stderr,_("error closing directory"));	
	}
	
	pop_status();	
	setNormalCursor();

}









void
on_recut1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}




void
on_filelist_click_column               (GtkCList        *clist,
                                        gint             column,
                                        gpointer         user_data)
{

}


gboolean
on_gfv_destroy_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  killApp();
  // gtk_main_quit();
  return FALSE;
}


/* Tree item selected */
void
on_tree1_selection_changed             (GtkTree         *tree,
                                        gpointer         user_data)
{
	GList * i;
	gchar * fname = NULL;
	GtkWidget * tentry = lookup_widget(mainApp,"fname");
	
	// g_print("selection changed for tree: %p\n",tree);
	// g_print("selected objects are:\n");
	
	push_status(_("Building directory tree"));
	setWaitCursor();
	i = GTK_TREE_SELECTION(tree);
	while (i){
	  gchar * name;
	  GtkLabel * l;
	  GtkWidget * w;
	  
	  w = GTK_WIDGET(i->data);
	  l = GTK_LABEL( GTK_BIN(w)->child);
	  gtk_label_get(l,&name);
	  // g_print("\t%s on level %d\n", name, GTK_TREE(w->parent)->level);
	  
	  fname = getTreeStringFull(GTK_TREE_ITEM(w), '/');
	  gtk_entry_set_text(GTK_ENTRY(tentry),fname);

	  if (cDir != NULL){
	  	g_free(cDir);
	  } 
	  cDir = g_strdup(fname);
	  
	  refreshFileList(fname);
	  
	  g_free(fname);
	  i = i->next;
	}
	pop_status();
	setNormalCursor();
	
}


/* Tree had a selection made */
void
on_tree1_select_child                  (GtkTree         *tree,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
}


/* Min app closing */
gboolean
on_gfv_delete_event                    (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data){

	// gtk_main_quit();                                        
	killApp();
	return TRUE;
}



void
on_settings_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	// on_base_directory1_activate(NULL,user_data);
  open_preference_window();
	
}


void
on_copy_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
	on_copy1_activate(NULL,user_data);

}


void
on_quit_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
  killApp();
  // gtk_main_quit();
}


void
on_about_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
	on_about1_activate(NULL, user_data);
}





void
on_filelist_resize_column              (GtkCList        *clist,
                                        gint             column,
                                        gint             width,
                                        gpointer         user_data)
{

  // Ok, lets handle the resize of the columns
  // We want to record this info in the config file
  gchar * sKey = NULL;


  sKey = g_strdup_printf("settings/FileListCol_%d",column);
  
  gnome_config_set_int(sKey, width);
  gnome_config_sync();
  
  g_free(sKey);
}

gboolean
on_filelist_button_press_event         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  gint r, c;

  if (event->button == SRL_RIGHT_BUTTON){
    if (gtk_clist_get_selection_info( GTK_CLIST(widget), event->x, event->y, &r, &c)){
      // -- Lets select the item in the clist
      gtk_clist_select_row(GTK_CLIST(widget), r, c); 
      // g_print("In click event!");
    }
  }
  
  
  return FALSE;
}


gboolean
on_tree1_button_press_event            (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  return FALSE;

}


void
on_select_all1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkText * txt = NULL;
  txt = GTK_TEXT(gtk_object_get_data (GTK_OBJECT (mainApp), "txtViewer"));
	
  gtk_editable_select_region( GTK_EDITABLE(txt), 0, -1);

}

gboolean on_eventbox1_button_release_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  // This is the location edit box....
  // If clicked, bring up the properties box ....
  if (event->button == 1){
    // on_base_directory1_activate(NULL,NULL);
    open_preference_window();

  }
  return FALSE;

}


/* ================== FIND =========================== */

// This is saved for future finds ... It also allows the "start from top" to have a little
// smarts.
static find_dlg_info * hInfo = NULL;

/* ----- Find function to be called when OK is pressed */
static void do_find_CALLBACK(find_dlg_info * info){

  GtkText *txt = GTK_TEXT(gtk_object_get_data (GTK_OBJECT (mainApp), "txtViewer"));

  int sPos = 0;
  gboolean first = TRUE;  
  gboolean fromStart = FALSE;

  // g_print("Find will be performed on: %s\n",info->text);
  
  if (hInfo != NULL){
    //g_print("H: %s,%d,%d\n",hInfo->text,hInfo->caseSensitive,hInfo->fromStart);
    //g_print("I: %s,%d,%d\n",info->text,info->caseSensitive,info->fromStart);    
    if (( g_strcasecmp(info->text, hInfo->text) == 0) && 
        ( info->fromStart == hInfo->fromStart) &&
        ( info->caseSensitive == hInfo->caseSensitive )) {
      first = FALSE;
    }
    g_free(hInfo);
  }
  hInfo = info;


  fromStart = hInfo->fromStart;
  if (!first){
    fromStart = FALSE;
    //g_print("This is not the first time\n");
  }

  
  if (!fromStart){
    sPos = gtk_editable_get_position(GTK_EDITABLE(txt));
    if (!first){		// Lets skip past the last item found
      sPos++;
    }
  }
  
  // g_print("Will search for %s from %d\n", hInfo->text, sPos);
  
  
  {
      // Get all the characters
      gchar * theText = gtk_editable_get_chars( GTK_EDITABLE(txt), sPos, -1);
      gchar * subStr = g_strdup(hInfo->text);
      gchar * result;
      
      // Convert both to uppercase is case insensitive search is being used
      if (theText == NULL) theText = g_malloc0(1);	// Safety for end
      
      if (!hInfo->caseSensitive){

        gchar * cc = theText;
        while (*cc){
          *cc = toupper(*cc);
          cc++;
        }
        
        cc = subStr;
        while (*cc){
          *cc = toupper(*cc);
          cc++;
        }
        
        
      }
      //g_print("Text: %s\n",theText);
      //g_print("Looking 4: %s\n", subStr);
      
      {
        result = strstr(theText,subStr);
        if (result == NULL){
          gtk_text_freeze(txt);
          gtk_text_set_point(txt,0);
          gtk_text_thaw(txt);          
          srlui_info("Search string not found");
        } else {
          // Get the offset of the text
          int offset = (int) (result - theText);
          offset += sPos;
          
          gtk_text_freeze(txt);
          gtk_editable_set_position(GTK_EDITABLE(txt), offset);
          gtk_editable_select_region(GTK_EDITABLE(txt), offset, offset+strlen(subStr));
          gtk_text_thaw(txt);          
        }
      }
      
      g_free(theText);
      g_free(subStr);
  }
  
}




void
on_find1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  open_find_window (do_find_CALLBACK);
}


/* This is for the find again button ... */
void
on_find_again1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  static find_dlg_info * temp = NULL;
  if (hInfo == NULL) return;
  
  temp = (find_dlg_info*) g_malloc(sizeof *temp);
  memcpy(temp,hInfo, sizeof *temp);
  do_find_CALLBACK(temp);

}


/* --- Gone .. all gone 
void
on_find_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
  on_find1_activate(NULL, NULL);
}
*/

void
on_btnFind_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  on_find1_activate(NULL, NULL);

}




void
on_dlgfnd_show                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
  _on_dlgfnd_show(widget,user_data);
}


void
on_find_text_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
  _on_find_text_changed(editable,user_data);

}


void
on_find_ok_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{

   _on_find_ok_clicked(button,user_data);
}


gboolean
on_dlgfnd_delete_event                 (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  return _on_dlgfnd_delete_event(widget,event,user_data);
}


void
on_find_cancel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{

  _on_find_cancel_clicked(button,user_data);
}

//================ END FIND =============================



// ============ POPUP MENU FOR THE TEXT EDIT WINDOW ========
void
on_find2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  on_find1_activate(NULL, NULL);  
}


void
on_find_again2_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  on_find_again1_activate(menuitem,user_data);
}

void
on_exit2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
   // gtk_widget_destroy(mainApp);
   // on_gfv_delete();
   killApp();

}


void
on_copy2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  on_copy1_activate(menuitem,user_data);
}


void
on_select_all2_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  on_select_all1_activate(menuitem,user_data);
}





// ======== END ==== POPUP MENU FOR THE TEXT EDIT WINDOW ========





  
  







/* ------------------- PREFERENCES DIALOG ! ------------------- */
void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  open_preference_window();
}



void
on_propertybox1_apply                  (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
  _on_propertybox1_apply(gnomepropertybox,arg1,user_data);

}



void
on_pref_down_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  _on_pref_down_clicked(button,user_data);
}


void
on_pref_loc_apply_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  _on_pref_loc_apply_clicked(button,user_data);
}


void on_pref_up_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  _on_pref_up_clicked(button,user_data);
}


void on_pref_delete_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  _on_pref_delete_clicked(button,user_data);
}



void on_pref_loc_entry_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
  _on_pref_loc_entry_changed(editable,user_data);
}


void on_pref_font_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  _on_pref_font_clicked(button,user_data);
}


void on_propertybox1_show                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
  _on_propertybox1_show(widget,user_data);
}



void on_locList_selection_changed           (GtkList         *list,
                                        gpointer         user_data)
{
  _on_locList_selection_changed(list,user_data);

}


void on_cb_expandTreeOnLoad_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

  _on_cb_expandTreeOnLoad_toggled(togglebutton,user_data);
}


/**** -------------- FILE LIST POPUP ------------------- ****/
void
on_all1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_filtered_1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_delete_file1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

  if (viewFile != NULL){
    gchar * q = g_strdup_printf("Are you sure you want to delete %s?", viewFile);
    GtkWidget * d = gnome_message_box_new(q,GNOME_MESSAGE_BOX_QUESTION,GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL,NULL);
    int rc = gnome_dialog_run_and_close(GNOME_DIALOG(d));
    if (rc == 0){
      int rr = unlink(viewFile);
      if (rr==-1){
        gchar * rslt = g_strdup_printf("Error deleting: %s\nError: %s", viewFile, strerror(errno));
        GtkWidget * gd = gnome_error_dialog (rslt);
        gnome_dialog_run_and_close( GNOME_DIALOG(gd) );
        g_free(rslt);
      } else {
        refreshFileList(cDir);
      }
    }
    g_free(q);
  }

}


void
on_edit_file1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gchar * cLine;
  int rc;

  if (viewFile == NULL) return;

  
  cLine = g_strdup_printf("%s %s &", editName, viewFile);
  rc=system(cLine);
  g_free(cLine);
  if (rc==-1){
    gchar * rslt = g_strdup_printf("Error processing: %s %s\nError: %s", editName, viewFile, strerror(errno));
    GtkWidget * gd = gnome_error_dialog (rslt);
    gnome_dialog_run_and_close( GNOME_DIALOG(gd) );
    g_free(rslt);
  } 
}
void on_edit2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  on_edit_file1_activate(menuitem,user_data);
}



void
on_preferences2_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  open_preference_window();
}





void
on_reload1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkText * txt = NULL;

  txt = GTK_TEXT(gtk_object_get_data (GTK_OBJECT (mainApp), "txtViewer"));

  if (viewFile != NULL){
    readFileIntoTextControl(txt , viewFile, gbl_font);
  }
}


void
on_editPrg_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  _on_editPrg_changed(editable,user_data);

}

