/*  - Starting Find Dialog Box
 * Copyright (C) 1999 Sam Lotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "find_dlg.h"
#include "srl_ui_util.h"
#include "support.h"

extern GtkWidget * mainApp;
GtkWidget * dlg_find = NULL;


static void (*callback)(find_dlg_info * info);

/*
  Note: The structure is allocated (g_malloc) before calling the callback.  It is up to the 
  called application to destroy this structure. 
*/
void open_find_window (void (*ok)(find_dlg_info * info)){
  if (dlg_find == NULL){
    callback = (gpointer) ok;
    dlg_find = create_dlgfnd();
    gnome_dialog_set_parent( GNOME_DIALOG(dlg_find), GTK_WINDOW(mainApp));

    gtk_window_set_policy(GTK_WINDOW(dlg_find),TRUE,TRUE,FALSE);  
    gtk_signal_connect( GTK_OBJECT(dlg_find), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dlg_find);
    gtk_widget_show (dlg_find);
    
  } else {
    gdk_window_show(dlg_find->window);
    gdk_window_raise(dlg_find->window);
  }
}


/* Is there already a find window openend?
*/
gboolean isOpen(void){
  return dlg_find != NULL;
}


/* DO NOT FREE */
static gchar * getTextEntered(){
  // Ok .. lets see if this is a valid directory....
  GtkWidget * eBox = NULL;
  
  // Ok, now set the text of the edit box
  eBox = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (dlg_find), "findText"));
  
  return gtk_entry_get_text(GTK_ENTRY(eBox));

}


static void enableButtons(){
  // Ok .. lets see if this is a valid directory....
  GtkWidget * okBtn = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (dlg_find), "btn_find_ok"));
  gchar * txt = NULL;

  txt = getTextEntered();  
  
  gtk_widget_set_sensitive	  (okBtn, strlen(txt)>0);
  
}



void _on_dlgfnd_show(GtkWidget *widget, gpointer user_data){

  GtkWidget * eBox = NULL;
  gnome_entry_set_history_id(GNOME_ENTRY( lookup_widget(dlg_find, "findText")->parent),"Find_Text");
  gnome_entry_set_max_saved   (GNOME_ENTRY( lookup_widget(dlg_find, "findText")->parent), 25);
  gnome_entry_load_history    (GNOME_ENTRY( lookup_widget(dlg_find, "findText")->parent));  
  
  
  // Ok, now set the text of the edit box
  eBox = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (dlg_find), "findText"));
  gtk_widget_grab_focus(eBox);
  enableButtons(); 
  
 
}

                                        
void _on_find_ok_clicked(GtkButton *button,gpointer user_data){
  find_dlg_info * info;
  
  info = (find_dlg_info*) g_malloc(sizeof *info);
  
  // Package up the information
  info->fromStart=(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
    lookup_widget(dlg_find, "cb_fromStart")))==TRUE);
  strncpy(info->text,getTextEntered(),MAXTEXT);
  info->text[MAXTEXT]='\0';
  info->caseSensitive=(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
    lookup_widget(dlg_find, "cb_case")))==TRUE);

  gnome_entry_save_history(GNOME_ENTRY( lookup_widget(dlg_find, "findText")->parent));  

  if (callback != NULL){
    (*callback)(info);
  }
}






void _on_find_text_changed(GtkEditable *editable,gpointer user_data){
  enableButtons();
}



gboolean _on_dlgfnd_delete_event(GtkWidget *widget,GdkEvent *event,gpointer user_data){
  return TRUE;
}


void _on_find_cancel_clicked(GtkButton *button,gpointer user_data){
   gtk_widget_destroy(dlg_find);
   dlg_find = NULL;
}





