/*  - Starting Preference Dialog Box
 * Copyright (C) 1999 Sam Lotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "pref_dlg.h"
#include "srl_ui_util.h"
#include "support.h"
#include "callbacks.h"

extern GtkWidget * mainApp;
GtkWidget * dlg_pref = NULL;
extern gchar *gbl_font_name;
extern gchar *editName;
extern GdkFont *gbl_font;

static gboolean doTree = FALSE;

static gboolean inLoad = FALSE;

/*** --- WINDOW PREP ---------*/
static void initList(){
  // For each item in the list ... lets find the child directories  
  GList* items = gtk_container_children(GTK_CONTAINER(lookup_widget(mainApp,"tree1")));
  GtkList * gls = GTK_LIST(lookup_widget(dlg_pref,"locList"));
  

  items = g_list_first(items);
  while (items != NULL){
  
    GtkWidget * w = NULL;
    gchar * lname;
    GtkTreeItem * ti = GTK_TREE_ITEM(items->data);
  
    GtkLabel * lbl = GTK_LABEL(GTK_BIN(ti)->child);
    gtk_label_get(lbl,&lname);  
  
  
    w  = gtk_list_item_new_with_label ( lname );
    // items = g_list_append(items, w  );
    gtk_container_add( GTK_CONTAINER(gls), w);
    gtk_widget_show(w);
    items = g_list_next(items);
    
  }
  
}
/* ------------------- */



// 	if (gnome_config_get_bool_with_default("settings/expand=false",&b)){


void open_preference_window (void){
  if (dlg_pref == NULL){
    doTree = FALSE;
  
    dlg_pref = create_propertybox1 ();
    gnome_dialog_set_parent( GNOME_DIALOG(dlg_pref), GTK_WINDOW(mainApp));
    gtk_window_set_title(GTK_WINDOW(dlg_pref),"Preferences");
    //gnome_dialog_set_parent( GNOME_DIALOG(dlg_pref), GTK_WINDOW(mainApp));
    gtk_window_set_policy(GTK_WINDOW(dlg_pref),TRUE,TRUE,FALSE);  
    gtk_signal_connect( GTK_OBJECT(dlg_pref), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dlg_pref);
    gtk_widget_show (dlg_pref);  
    
  } else {
    gdk_window_show(dlg_pref->window);
    gdk_window_raise(dlg_pref->window);
  }
}


static void set_font_entry(const gchar * val){
  GtkWidget * eBox = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (dlg_pref), "font_entry"));
  gchar * cVal = gtk_entry_get_text(GTK_ENTRY(eBox));
  
  gtk_entry_set_text(GTK_ENTRY(eBox), val);
  if (strcmp(val, cVal) != 0){
    if (!inLoad){
      gnome_property_box_changed( GNOME_PROPERTY_BOX(dlg_pref));
    }
  }
}


static gchar *  get_font_entry(void){
  GtkWidget * eBox = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (dlg_pref), "font_entry"));
  gchar * cVal = gtk_entry_get_text(GTK_ENTRY(eBox));
  // g_print("Font in edit box: %s\n", cVal);
  return cVal;  
}



/* ----------------------------------- ENABLE BUTTONS ROUTINE --------------------- */

// DO NOT FREE POINTER
static gchar * getAddLoc(){
  // Ok .. lets see if this is a valid directory....
  GtkWidget * eBox = NULL;
  
  // Ok, now set the text of the edit box
  eBox = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (dlg_pref), "loc_entry"));
  return gtk_entry_get_text(GTK_ENTRY(eBox));

}


static gboolean islocDirValid(){
  return isDirectory(getAddLoc());
}




static void enableButtons_LOCPAGE(){
  GtkList * lst = GTK_LIST(gtk_object_get_data (GTK_OBJECT (dlg_pref), "locList"));
  GtkButton * up = GTK_BUTTON(gtk_object_get_data (GTK_OBJECT (dlg_pref), "loc_up"));
  GtkButton * down = GTK_BUTTON(gtk_object_get_data (GTK_OBJECT (dlg_pref), "loc_down"));
  GtkButton * delete = GTK_BUTTON(gtk_object_get_data (GTK_OBJECT (dlg_pref), "loc_delete"));
  
  // Ok .. lets see if this is a valid directory....
  GtkWidget * okBtn = GTK_WIDGET(gtk_object_get_data (GTK_OBJECT (dlg_pref), "loc_apply"));
  gtk_widget_set_sensitive	  (okBtn, (islocDirValid()));

  // Ok ... Do we have anything in the list .. selected?
  if (lst->selection == NULL){
     gtk_widget_set_sensitive(GTK_WIDGET(up),FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(down),FALSE);
     gtk_widget_set_sensitive(GTK_WIDGET(delete),FALSE);          
  } else {
     gtk_widget_set_sensitive(GTK_WIDGET(up),TRUE);
     gtk_widget_set_sensitive(GTK_WIDGET(down),TRUE);
     gtk_widget_set_sensitive(GTK_WIDGET(delete),TRUE);          
  }     



  
}






static void enableButtons(){
  enableButtons_LOCPAGE();
}
/* ----------------------------------- ========================== --------------------- */



void _on_propertybox1_apply(GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data){


  int cnt = 0;
  char buf[50];
  gchar * fnt = get_font_entry();

  // g_print("Apply called %d\n", arg1);
  if (arg1 == -1){
    GList* dlist = gtk_container_children(GTK_CONTAINER(lookup_widget(dlg_pref,"locList")));
    GtkEntry * entry = GTK_ENTRY(lookup_widget(dlg_pref,"editPrg"));

    GtkToggleButton * expandTree = GTK_TOGGLE_BUTTON(lookup_widget(dlg_pref,"cb_expandTreeOnLoad"));
    gnome_config_set_bool("settings/expand", gtk_toggle_button_get_active(expandTree));
    gnome_config_sync();

    // Get the edit name
    g_free(editName);
    editName = gtk_entry_get_text(entry);
    editName = g_strdup(editName);
    gnome_config_set_string("settings/editName",editName);
    gnome_config_sync();
    
  
    
    // Setup the default base ... we do it here bacuse of callback from dialog
    //gnome_config_set_string("settings/BASE_DIR",base);
    
    
    if (doTree){
      killEntireTree();
      while (dlist) {
        GtkObject       *list_item;
        gchar           *item_data_string;
        GtkLabel * b;
	

        list_item=GTK_OBJECT(dlist->data);
        b=GTK_LABEL(GTK_BIN(list_item)->child);
        gtk_label_get(b,&item_data_string);  

        // Record in config
        sprintf(buf,"settings/BASE_DIR_ENTRY%d",cnt);
        gnome_config_set_string(buf,item_data_string);
        cnt++;

        buildInitialTree(item_data_string);      
        // g_print("Dir: %s\n", item_data_string);  
	
        dlist=dlist->next;
      }
      gnome_config_set_int("settings/BDCount",cnt);

    }
    
    // Setup the font 
    if (strlen(fnt) != 0){
      if (gbl_font_name != NULL) g_free(gbl_font_name);
      if (gbl_font != NULL) gdk_font_unref(gbl_font);

      gbl_font_name = g_strdup(fnt);
      gbl_font = gdk_font_load(gbl_font_name);
      gnome_config_set_string("settings/font",gbl_font_name);
      
      refreshFileView();

    }   

    gnome_config_sync();
    
  }
}








void _on_pref_down_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
}



void _on_pref_up_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  enableButtons();
}



/** --- DELETe AN ITEM FROM THE LIST */
void _on_pref_delete_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkList * lst = GTK_LIST(gtk_object_get_data (GTK_OBJECT (dlg_pref), "locList"));
  GList * item_list = NULL;
  
  item_list = g_list_alloc ();
  item_list->data = ((GList*)(GTK_LIST(lst)->selection))->data;
  
  gtk_list_remove_items (GTK_LIST (lst), item_list);
  
  g_list_free (item_list);  

  enableButtons();
  gnome_property_box_changed( GNOME_PROPERTY_BOX(dlg_pref));
  
}




void _on_pref_loc_entry_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
  enableButtons();
}






void _on_pref_loc_apply_clicked(GtkButton *button, gpointer user_data)
{
  GtkList * gls = GTK_LIST(lookup_widget(dlg_pref,"locList"));
  GtkWidget * w = NULL;
  gchar * theEntry = NULL;
  
  theEntry = getAddLoc();
  
  // Add a / if needed .. also make it a malloced storage area
  if (theEntry[ strlen(theEntry)-1 ] != '/'){
    theEntry = g_strdup_printf("%s/", theEntry);
  } else {
    theEntry = g_strdup_printf("%s", theEntry);
  }
  
  w  = gtk_list_item_new_with_label ( theEntry );
  // items = g_list_append(items, w  );
  gtk_container_add( GTK_CONTAINER(gls), w);
  gtk_widget_show(w);
  
  g_free(theEntry);

  doTree = TRUE;  
  gnome_property_box_changed( GNOME_PROPERTY_BOX(dlg_pref));
}




void _on_pref_font_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget* fsd, *dialog;
  int rc;
  gchar * f=get_font_entry();

  dialog = gnome_dialog_new(_("Font Selection"), 
           GNOME_STOCK_BUTTON_OK,
           GNOME_STOCK_BUTTON_CANCEL,
           NULL);
           
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
  	(fsd=gtk_font_selection_new()),
  	TRUE,TRUE,0);
  	
  if (f!=NULL){
    gtk_font_selection_set_font_name(GTK_FONT_SELECTION(fsd),f); 
  }

  if (gbl_font_name != NULL){
    gtk_font_selection_set_font_name(GTK_FONT_SELECTION(fsd),gbl_font_name);
  }
 	
  gtk_widget_show(fsd);
  gnome_dialog_set_default(GNOME_DIALOG(dialog),GNOME_OK);
  gnome_dialog_close_hides(GNOME_DIALOG(dialog),TRUE);
  gnome_dialog_set_parent( GNOME_DIALOG(dialog), GTK_WINDOW(mainApp));
  rc = gnome_dialog_run( GNOME_DIALOG(dialog) );
  if (rc == GNOME_OK){
    gchar * myFont = gtk_font_selection_get_font_name(GTK_FONT_SELECTION(fsd));    
    if (myFont != NULL){
      set_font_entry(myFont);
      g_free(myFont);    
    }
  }
  gtk_widget_destroy(dialog);

}

                                        
void _on_propertybox1_show                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
  gboolean b;
  
  GtkEntry * entry = GTK_ENTRY(lookup_widget(dlg_pref,"editPrg"));
  GtkToggleButton * expandTree = GTK_TOGGLE_BUTTON(lookup_widget(dlg_pref,"cb_expandTreeOnLoad"));

  inLoad = TRUE;  
  initList();
  if (gbl_font_name != NULL){
    set_font_entry(gbl_font_name);
  }

  gtk_toggle_button_set_active(expandTree, 
     gnome_config_get_bool_with_default("settings/expand=false", &b));

  gtk_entry_set_text(entry,editName);

  inLoad = FALSE;  
}

void _on_locList_selection_changed           (GtkList         *list,
                                        gpointer         user_data){
  enableButtons();
}                                        


void _on_cb_expandTreeOnLoad_toggled(GtkToggleButton *togglebutton,gpointer user_data)
{
  if (!inLoad ){
    gnome_property_box_changed( GNOME_PROPERTY_BOX(dlg_pref));
  }
}


void _on_editPrg_changed                     (GtkEditable     *editable,
                                        gpointer         user_data){

  if (!inLoad ){
    gnome_property_box_changed( GNOME_PROPERTY_BOX(dlg_pref));
  }

}                                        

