/* GFV User Interface Utility Functions
 * Copyright (C) 1999 Sam Lotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "srl_ui_util.h"

#include <sys/stat.h>
#include <sys/types.h>


void srlui_info(char * msg){
	GtkWidget * box;
	box = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_INFO, GNOME_STOCK_BUTTON_OK, NULL);
	// gtk_widget_show(box);
	
	gnome_dialog_run( GNOME_DIALOG(box) );
	
}



/* ------------------------------------------ */
extern GtkWidget * mainApp;

static int waitCount = 0;

void setWaitCursor(void){
   GdkCursor * c = gdk_cursor_new(GDK_WATCH);
   gdk_window_set_cursor( GTK_WIDGET(mainApp)->window, c);
   gdk_cursor_destroy(c);
   waitCount++;
}

void setNormalCursor(void){
   if (--waitCount == 0){
     gdk_window_set_cursor( GTK_WIDGET(mainApp)->window, NULL);
   }
}



void gui_update(void){
   while (gtk_events_pending()){
     gtk_main_iteration();
   }
}

void push_status(char * st){
   GnomeAppBar * appBar = NULL;
   appBar = GNOME_APPBAR(gtk_object_get_data (GTK_OBJECT (mainApp), "appbar1"));
   gnome_appbar_push(appBar,st);
   gui_update();
}

void pop_status(void){
   GnomeAppBar * appBar = NULL;
   appBar = GNOME_APPBAR(gtk_object_get_data (GTK_OBJECT (mainApp), "appbar1"));
   gnome_appbar_pop(appBar);
   gui_update();
}
/* --------------------------------------------- */



/* Return true if this is a directory */
int isDirectory(const char * dir){
  struct stat buf;
  
  stat(dir,&buf);
  return (buf.st_mode & S_IFDIR);
}


/* Return true if this is a directory */
int isDirectory2(const char * dir, const char * child){
  char *b;
  gboolean rslt;
  b= g_strdup_printf("%s%s",dir,child);
  rslt =  isDirectory(b);
  g_free(b);
  return rslt;
}


struct stat *  getStat2(const char * dir, const char * name){
  char *b;
  struct stat * st;
  
  b= g_strdup_printf("%s%s",dir,name);
  st = g_malloc(sizeof( struct stat));
  stat(b,st);
  g_free(b);  
  return st;

}



/* For every tree item .. lets register and handle the expand event */
static void treeitem_expand(GtkWidget * item, gpointer data){
	gchar * cc1;
	// Lets get all the entries in this tree and build their children
	GList * l;
	
	//return;
	if ((GTK_TREE_ITEM(item)->subtree)==NULL) return;
	
	
	push_status(_("Expanding tree ... please wait"));
	setWaitCursor();

	
	
	// cc1 = getTreeStringFull(GTK_TREE_ITEM(item), '/');
	// g_print("expanding: %s\n", cc1);
  	// g_free(cc1);
	
	
	
	l=gtk_container_children( GTK_CONTAINER( GTK_TREE_ITEM (item)->subtree));
	
	if (l==NULL){
	  pop_status();	
	  setNormalCursor();
	  return;
	}
	
	/* For each tree item .. add a subtree (if directories exists under it) */
	while(l != NULL){
	  
	  cc1 = getTreeStringFull(GTK_TREE_ITEM(l->data), '/');
	  
	  //g_print("Building 4: %s/n", cc1);
	  if ((GTK_TREE_ITEM(l->data)->subtree)!=NULL){
	    gtk_tree_item_remove_subtree (GTK_TREE_ITEM(l->data)); 
	  }
	  buildTreeFromDir(GTK_TREE_ITEM(l->data), cc1);
  	  g_free(cc1);

	  
	  l=l->next;
	}
	pop_status();
	setNormalCursor();
	
	
}


/* 1 level of recursion to supply the + or - for directories at the current level */
static GtkTree * _buildTreeFromDir(GtkTreeItem * parent, char * dirname, int recurse){
	GtkTree * st = NULL;
	GtkTreeItem * sitem = NULL;
	DIR *dir = opendir(dirname);
	struct dirent *ent = NULL;

        if (dir == NULL){
	  return NULL;
        }
	while( (ent = readdir(dir)) != NULL){
	  if (strcmp(ent->d_name,".") == 0) continue;
	  if (strcmp(ent->d_name,"..") == 0) continue;
	  
	  if (isDirectory2(dirname, ent->d_name)){
	    
	    if (st == NULL) st = GTK_TREE(gtk_tree_new());
	    
	    sitem = GTK_TREE_ITEM(gtk_tree_item_new_with_label(ent->d_name));
	    gtk_tree_append( st, GTK_WIDGET(sitem));
	    gtk_widget_show( GTK_WIDGET(sitem) );
	    gtk_signal_connect( GTK_OBJECT(sitem),"expand",GTK_SIGNAL_FUNC(treeitem_expand),NULL);
	    

	  } 
	}
	closedir(dir);

	if (st != NULL){
	  gtk_widget_show( GTK_WIDGET(st));	
	  gtk_tree_item_set_subtree(parent,GTK_WIDGET(st));
	
	  if (recurse){
	  	    
	    // For each item in the list ... lets find the child directories
	    GList* items = gtk_container_children(GTK_CONTAINER(st));

	    items = g_list_first(items);
	    while (items != NULL){
	      char pname[256];
	      gchar * lname;
	      GtkTreeItem * ti = GTK_TREE_ITEM(items->data);
	      GtkLabel * lbl = GTK_LABEL(GTK_BIN(ti)->child);
	      gtk_label_get(lbl,&lname);
	      g_snprintf(pname,256,"%s%s/", dirname, lname);
	      _buildTreeFromDir(ti, pname, FALSE);
	    
	      items = g_list_next(items);
	    }
	  }
	}

	return st;
}


GtkTree * buildTreeFromDir(GtkTreeItem * parent, char * dir){
	return _buildTreeFromDir(parent, dir,TRUE);
}



/*
	Returns the string at the currently selected GTK_TREE
	item.
	
	it iterates through the parents, appending the
	appending sctring between then
	
	Must g_free the memory returned
*/
gchar * getTreeStringFull(GtkTreeItem * item, gchar sep){
	char *rslt = g_malloc0(512);
	char * tmp = g_malloc0(512);
	int levels = 0;
	char csep[2];
	
	gchar * name;
	GtkLabel * l;
	GtkWidget * w;
	
	csep[0]=sep; csep[1]='\0';
	
	w = GTK_WIDGET(item);
	levels = (GTK_TREE(w->parent)->level);
	
	// W is always the current item
	// rslt is always the current string
	
	//g_print("-----\n");
	while (levels>=0){
  	  l = GTK_LABEL( GTK_BIN(w)->child);
	  gtk_label_get(l,&name);
	  
	  //g_print("Level: %d, name: %s\n",levels,name);
	  
	  if (name[strlen(name)-1] != sep){
	     sprintf(tmp,"%s%c",name,sep);
	  } else {
	     sprintf(tmp,"%s",name);
	  }
	  strcat(tmp,rslt);
	  
	  strcpy(rslt,tmp);
	  
	  levels--;
	  if (levels > -1){
	    w = w->parent;		// Tree
    	    /* if (GTK_IS_TREE(w)){
	      g_print("tree as expected\n");
	    } */

	    w = GTK_TREE(w)->tree_owner;		// Tree item
   	    /* if (GTK_IS_TREE_ITEM(w)){
	      g_print("treeitem as expected\n");
	    }else {
	      g_print("not a treeitem\n");
	    } */
	    
	  }
	   
	}
	g_free(tmp);
	return rslt;
	
}





/* Read a file and append it into a text box
   Returns TRUE if OK
   	FALSE if Error
*/
gboolean readFileIntoTextControl(GtkText * cntl, gchar * fname, GdkFont * fnt){

	FILE * inf = NULL;
	gchar * buffer = NULL;
	
	/* Remove all text in the control */
	gtk_text_freeze(cntl);
	gtk_editable_delete_text(GTK_EDITABLE(cntl), 0, gtk_text_get_length(cntl));
	
	//g_print(fname);
	inf = fopen(fname, "r");
	if (inf == NULL){
	  gtk_text_thaw(cntl);
	  return FALSE;
	}
	
	
	buffer = g_malloc(8193);
	while ( (fgets(buffer, 8192, inf))!=NULL ){
	  // g_print(buffer);
	  gtk_text_insert(cntl, fnt, NULL, NULL, buffer, strlen(buffer));
	}
	fclose(inf);
	g_free(buffer);
	
	gtk_text_set_point(cntl,0);
	gtk_text_thaw(cntl);
	return TRUE;
}

